@@ -11,13 +11,14 @@ except ImportError:
     # NOTE - could use subprocess to get from git?
     __rev__ = "local"
 
-__version__ = "0.12.0"
+__version__ = "0.12.2"
 
 _TEST_ENV = bool(os.environ.get("DP_TEST_ENV", ""))
 _IN_PYTEST = "pytest" in sys.modules
 _IN_DPSERVER = "dp" in sys.modules
 # we're running on datapane platform
 ON_DATAPANE: bool = "DATAPANE_ON_DATAPANE" in os.environ
+_USING_CONDA = os.path.exists(os.path.join(sys.prefix, "conda-meta", "history"))
 
 
 # Other useful re-exports
@@ -53,10 +54,13 @@ from .client.api import (
     TextAlignment,
     TextReport,
     Variable,
+    Visibility,
     by_datapane,
+    signup,
     login,
     logout,
     ping,
+    hello_world,
     builtins,
 )
 from .client.config import init
