@@ -3,11 +3,11 @@ import platform
 from contextlib import suppress
 from functools import wraps
 from pathlib import Path
-from typing import Optional
 
 import posthog
 
-from datapane import _IN_DPSERVER, _IN_PYTEST, ON_DATAPANE, __version__, log
+from datapane import _IN_DPSERVER, _IN_PYTEST, _USING_CONDA, ON_DATAPANE, __version__, log
+from datapane.client.utils import is_jupyter
 
 from . import config as c
 
@@ -28,31 +28,43 @@ def is_analytics_disabled() -> bool:
 _NO_ANALYTICS: bool = is_analytics_disabled()
 
 
-def capture(event: str, properties: Optional[dict] = None) -> None:
+def capture(event: str, **properties) -> None:
     # Used for capturing generic events with properties
     if _NO_ANALYTICS:
         return None
     config = c.get_config()
-    properties = properties or {}
-    properties.update(source="cli", dp_version=__version__)
+
+    # run identify on first action, (NOTE - don't change the order here)
+    if not config.completed_action:
+        config.completed_action = True
+        identify(config.session_id)
+        config.save()
+
+    properties.update(source="cli", dp_version=__version__, in_jupyter=is_jupyter(), using_conda=_USING_CONDA)
+
     with suppress(Exception):
         posthog.capture(config.session_id, event, properties)
 
 
-def identify(session_id: str, properties: Optional[dict] = None) -> None:
-    properties = properties or {}
-    properties.update(os=platform.system(), python_version=platform.python_version(), dp_version=__version__)
+def identify(session_id: str, **properties) -> None:
+    properties.update(
+        os=platform.system(),
+        python_version=platform.python_version(),
+        dp_version=__version__,
+        in_jupyter=is_jupyter(),
+        using_conda=_USING_CONDA,
+    )
     with suppress(Exception):
         posthog.identify(session_id, properties)
     # Also generate a CLI identify event to help disambiguation
     capture("CLI Identify")
 
 
-def capture_init(config: c.Config) -> None:
-    # Generates an identify event on init
-    if _NO_ANALYTICS:
-        return None
-    identify(config.session_id)
+# def capture_init(config: c.Config) -> None:
+#     # Generates an identify event on init
+#     if _NO_ANALYTICS:
+#         return None
+#     identify(config.session_id)
 
 
 def capture_event(name: str):
