@@ -24,7 +24,7 @@ from lxml.etree import Element
 from pandas.io.formats.style import Styler
 
 from datapane.client import DPError
-from datapane.common import PKL_MIMETYPE, NPath, guess_type, utf_read_text
+from datapane.common import PKL_MIMETYPE, NPath, guess_type, log, utf_read_text
 from datapane.common.report import conv_attribs, get_embed_url, is_valid_id
 
 from ..common import DPTmpFile
@@ -108,10 +108,17 @@ class BaseElement(ABC):
         self._add_attributes(**kwargs)
         self._set_name(name)
 
-        if "caption" in kwargs:
-            _caption = kwargs["caption"]
-            if _caption and len(_caption) > 512:
-                raise DPError("Caption must be less than 512 characters")
+        self._truncate_strings(kwargs, "caption", 512)
+        self._truncate_strings(kwargs, "label", 256)
+
+    def _truncate_strings(self, kwargs: dict, key: str, max_length: int):
+        if key in kwargs:
+            x: str = kwargs[key]
+            if x and len(x) > max_length:
+                kwargs[key] = f"{x[:max_length-3]}..."
+                log.warning(f"{key} currently '{x}'")
+                log.warning(f"{key} must be less than {max_length} characters, truncating")
+                # raise DPError(f"{key} must be less than {max_length} characters, '{x}'")
 
     def _set_name(self, name: str = None):
         if name:
@@ -299,7 +306,7 @@ class Group(LayoutBlock):
             rows: Display the contained blocks, e.g. Plots, using _n_ rows (set to 0 to autofill)
             columns: Display the contained blocks, e.g. Plots, using _n_ columns (set to 0 to autofill)
 
-        .. tip:: Group can be passed using either arg parameters or the `blocks` kwarg, e.g.
+        ..tip:: Group can be passed using either arg parameters or the `blocks` kwarg, e.g.
           `dp.Group(plot, table, columns=2)` or `dp.Group(blocks=[plot, table], columns=2)`
         """
         super().__init__(*arg_blocks, blocks=blocks, name=name, label=label)
@@ -337,6 +344,10 @@ class EmbeddedTextBlock(DataBlock):
 
     content: str
 
+    def __init__(self, content: str, name: str = None, **kwargs):
+        super().__init__(name, **kwargs)
+        self.content = content.strip()
+
     def _to_xml(self, s: BuilderState) -> BuilderState:
         # NOTE - do we use etree.CDATA wrapper?
         _E = getattr(E, self._tag)
@@ -360,15 +371,16 @@ class Text(EmbeddedTextBlock):
             text: The markdown formatted text, use triple-quotes, (`\"\"\"# My Title\"\"\"`) to create multi-line markdown text
             file: Path to a file containing markdown text
             name: A unique name for the block to reference when adding text or embedding (optional)
+            label: A label used when displaying the block (optional)
 
         ..note:: File encodings are auto-detected, if this fails please read the file manually with an explicit encoding and use the text parameter on dp.File
         """
-        super().__init__(name=name, label=label)
-
         if text:
             text = text.strip()
+
         assert text or file
-        self.content = text or utf_read_text(Path(file).expanduser())
+        content = text or utf_read_text(Path(file).expanduser())
+        super().__init__(content=content, name=name, label=label)
 
     def format(self, *args: BlockOrPrimitive, **kwargs: BlockOrPrimitive) -> Group:
         """
@@ -426,8 +438,7 @@ class Code(EmbeddedTextBlock):
             name: A unique name for the block to reference when adding text or embedding (optional)
             label: A label used when displaying the block (optional)
         """
-        super().__init__(language=language, caption=caption, name=name, label=label)
-        self.content = code
+        super().__init__(content=code, language=language, caption=caption, name=name, label=label)
 
 
 class HTML(EmbeddedTextBlock):
@@ -444,8 +455,7 @@ class HTML(EmbeddedTextBlock):
             name: A unique name for the block to reference when adding text or embedding (optional)
             label: A label used when displaying the block (optional)
         """
-        super().__init__(name=name, label=label)
-        self.content = str(html)
+        super().__init__(content=str(html), name=name, label=label)
 
 
 class Formula(EmbeddedTextBlock):
@@ -466,9 +476,7 @@ class Formula(EmbeddedTextBlock):
             ..note:: LaTeX commonly uses special characters, hence prefix your formulas with `r` to make them
             raw strings, e.g. r"\frac{1}{\sqrt{x^2 + 1}}"
         """
-
-        super().__init__(caption=caption, name=name, label=label)
-        self.content = formula
+        super().__init__(content=formula, caption=caption, name=name, label=label)
 
 
 class Embed(EmbeddedTextBlock):
@@ -489,11 +497,12 @@ class Embed(EmbeddedTextBlock):
         """
 
         result = get_embed_url(url, width=width, height=height)
-        super().__init__(name=name, label=label, url=url, title=result.title, provider_name=result.provider)
 
         # if "html" not in result:
         #     raise DPError(f"Can't embed result from provider for URL '{url}'")
-        self.content = result.html
+        super().__init__(
+            content=result.html, name=name, label=label, url=url, title=result.title, provider_name=result.provider
+        )
 
 
 NumberValue = t.Union[str, int, float]
@@ -617,6 +626,7 @@ class File(AssetBlock):
         file: t.Optional[NPath] = None,
         is_json: bool = False,
         filename: t.Optional[str] = None,
+        caption: t.Optional[str] = None,
         name: str = None,
         label: str = None,
     ):
@@ -626,6 +636,7 @@ class File(AssetBlock):
             file: Path to a file to attach to the report (e.g. a JPEG image)
             is_json: If `True`, treat the `data` as JSON data already
             filename: Name to be used when downloading the file
+            caption: A caption to display below the file (optional)
             name: A unique name for the block to reference when adding text or embedding (optional)
             label: A label used when displaying the block (optional)
 
@@ -637,7 +648,7 @@ class File(AssetBlock):
             out_fn = self._save_obj(data, as_json=is_json)
             file = out_fn.file
 
-        super().__init__(file=file, filename=filename or file.name, name=name, label=label)
+        super().__init__(file=file, filename=filename or file.name, caption=caption, name=name, label=label)
 
 
 class Plot(AssetBlock):
