@@ -4,12 +4,11 @@ Datapane Reports Object
 Describes the `Report` object and included APIs for saving and uploading them.
 """
 import dataclasses as dc
-import json
 import random
 import typing as t
 import webbrowser
 from base64 import b64encode
-from enum import Enum
+from enum import Enum, IntEnum
 from functools import reduce
 from os import path as osp
 from pathlib import Path
@@ -21,11 +20,11 @@ from lxml import etree
 from lxml.etree import Element
 
 from datapane.client import config as c
-from datapane.client.analytics import capture_event
+from datapane.client.analytics import _NO_ANALYTICS, capture, capture_event
 from datapane.client.api.common import DPTmpFile, Resource
 from datapane.client.api.dp_object import DPObjectRef
 from datapane.client.api.runtime import _report
-from datapane.client.utils import DPError, InvalidReportError, is_jupyter
+from datapane.client.utils import DPError, InvalidReportError, display_msg
 from datapane.common import NPath, dict_drop_empty, log, timestamp
 from datapane.common.report import local_report_def, validate_report_doc
 from datapane.common.utils import process_notebook
@@ -48,13 +47,24 @@ local_post_xslt = etree.parse(str(local_report_def / "local_post_process.xslt"))
 local_post_transform = etree.XSLT(local_post_xslt)
 
 # only these types will be documented by default
-__all__ = ["Report", "ReportWidth"]
+__all__ = ["Report", "ReportWidth", "Visibility"]
 
 __pdoc__ = {
     "Report.endpoint": False,
 }
 
 
+class Visibility(IntEnum):
+    """The report visibility type, set for reports on Datapane Studio
+
+    ..note:: This is ignored on Datapane Enterprise, reports are always private
+    """
+
+    DEFAULT = 1  # not listed on the users portfolio
+    PORTFOLIO = 2  # above + added to user's portfolio page
+    PRIVATE = 3  # private to owner
+
+
 class ReportWidth(Enum):
     """The document width"""
 
@@ -79,7 +89,7 @@ class FontChoice(Enum):
 
 @dc.dataclass
 class ReportFormatting:
-    """Set the basic styling for your report"""
+    """Sets the report styling and formatting"""
 
     bg_color: str = "#FFF"
     accent_color: str = "#4E46E5"
@@ -103,17 +113,7 @@ class ReportFormatting:
 
 
 # Used to detect a single display message once per VM invocation
-SKIP_DISPLAY_MSG = False
-
-
-def display_msg(text: str, md: str = None):
-    if is_jupyter():
-        from IPython.display import Markdown, display
-
-        msg = md or text
-        display(Markdown(msg))
-    else:
-        print(text)
+# SKIP_DISPLAY_MSG = False
 
 
 @contextfunction
@@ -157,7 +157,7 @@ class ReportFileWriter:
         name: str,
         author: t.Optional[str] = None,
         formatting: ReportFormatting = None,
-    ):
+    ) -> str:
 
         if formatting is None:
             formatting = ReportFormatting()
@@ -166,14 +166,9 @@ class ReportFileWriter:
         if not self.template:
             self._setup_template()
 
-        url = "https://datapane.com/teams/"
-        display_msg(
-            text=f"Report saved to {path}. To host and share securely, request a free private workspace at {url}",
-            md=f"Report saved to {path}. To host and share securely, [request a free private workspace]({url})",
-        )
-
         # template.html inlines the report doc with backticks so we need to escape any inside the doc
         report_doc_esc = report_doc.replace("`", r"\`")
+        report_id = uuid4().hex
         r = self.template.render(
             report_doc=report_doc_esc,
             report_width=formatting.width.value,
@@ -181,11 +176,16 @@ class ReportFileWriter:
             report_author=author,
             report_date=timestamp(),
             css_header=formatting.to_css(),
-            is_light_prose=json.dumps(formatting.light_prose),
+            is_light_prose=formatting.light_prose,
             dp_logo=self.logo,
+            report_id=report_id,
+            author_id=c.config.session_id,
+            analytics=not _NO_ANALYTICS,
         )
         Path(path).write_text(r, encoding="utf-8")
 
+        return report_id
+
 
 class BaseReport(DPObjectRef):
     endpoint: str = "/reports/"
@@ -249,6 +249,7 @@ class BaseReport(DPObjectRef):
         name: str,
         description: str = "",
         source_url: str = "",
+        visibility: t.Union[Visibility, str] = "",
         tags: t.List[str] = None,
         group: t.Optional[str] = None,
         source_file: t.Optional[NPath] = None,
@@ -258,8 +259,9 @@ class BaseReport(DPObjectRef):
         # TODO - clean up arg handling
         # process params
         tags = tags or []
-        formatting_kwargs = {}
+        visibility_str = visibility.upper() if isinstance(visibility, str) else visibility.name
 
+        formatting_kwargs = {}
         if formatting:
             formatting_kwargs.update(
                 width=formatting.width.value,
@@ -276,6 +278,7 @@ class BaseReport(DPObjectRef):
             description=description,
             tags=tags,
             source_url=source_url,
+            visibility=visibility_str,
             group=group,
             **formatting_kwargs,
         )
@@ -327,10 +330,10 @@ class TextReport(BaseReport):
         Returns:
             A `TextReport` object containing assets that can be uploaded for use with your online TextReport
 
-        .. tip:: Blocks can be passed using either arg parameters or the `blocks` kwarg as a dictionary, e.g.
+        ..tip:: Blocks can be passed using either arg parameters or the `blocks` kwarg as a dictionary, e.g.
           `dp.TextReport(my_plot=plot, my_table=table)` or `dp.TextReport(blocks={"my_plot": plot, "my_table":table})`
 
-        .. tip:: Create a dictionary first to hold your blocks to edit them dynamically, for instance when using Jupyter, and use the `blocks` parameter
+        ..tip:: Create a dictionary first to hold your blocks to edit them dynamically, for instance when using Jupyter, and use the `blocks` parameter
         """
         super().__init__()
 
@@ -366,6 +369,7 @@ class TextReport(BaseReport):
         name: t.Optional[str] = "",
         description: str = "",
         source_url: str = "",
+        visibility: t.Union[Visibility, str] = "",
         tags: t.List[str] = None,
         group: t.Optional[str] = None,
         source_file: t.Optional[NPath] = None,
@@ -381,6 +385,7 @@ class TextReport(BaseReport):
             name: The document name - can include spaces, caps, symbols, etc., e.g. "Profit & Loss 2020"
             description: A high-level description for the document, this is displayed in searches and thumbnails
             source_url: A URL pointing to the source code for the document, e.g. a GitHub repo or a Colab notebook
+            visibility: "UNLISTED" (default), "PRIVATE", or "PUBLISHED" (for studio product only, ignored on enterprise)
             tags: A list of tags (as strings) used to categorise your document
             group: Group to add the report to (Teams only)
             source_file: Path of jupyter notebook file to upload
@@ -403,6 +408,7 @@ class TextReport(BaseReport):
             name,
             description,
             source_url,
+            visibility,
             tags,
             group,
             source_file,
@@ -439,12 +445,10 @@ class Report(BaseReport):
     can be analysed and shared by users in their Browser
     """
 
-    _last_saved: t.Optional[str] = None  # Path to local report
     _tmp_report: t.Optional[Path] = None  # Temp local report
     _local_writer = ReportFileWriter()
     _preview_file: str = DPTmpFile(f"{uuid4().hex}.html")
     list_fields: t.List[str] = ["name", "web_url", "group"]
-    """When set, the report is full-width suitable for use in a dashboard"""
 
     def __init__(
         self,
@@ -456,15 +460,14 @@ class Report(BaseReport):
         Args:
             *arg_blocks: Group to add to document
             blocks: Allows providing the document blocks as a single list
-            type: Set the Report type, this will affect the formatting and layout of the document
 
         Returns:
             A `Report` document object that can be uploaded, saved, etc.
 
-        .. tip:: Blocks can be passed using either arg parameters or the `blocks` kwarg, e.g.
+        ..tip:: Blocks can be passed using either arg parameters or the `blocks` kwarg, e.g.
           `dp.Report(plot, table)` or `dp.Report(blocks=[plot, table])`
 
-        .. tip:: Create a list first to hold your blocks to edit them dynmically, for instance when using Jupyter, and use the `blocks` parameter
+        ..tip:: Create a list first to hold your blocks to edit them dynamically, for instance when using Jupyter, and use the `blocks` parameter
         """
         super().__init__(**kwargs)
         self._preprocess_pages(blocks or list(arg_blocks))
@@ -486,6 +489,7 @@ class Report(BaseReport):
         name: str,
         description: str = "",
         source_url: str = "",
+        visibility: t.Union[Visibility, str] = "",
         tags: t.List[str] = None,
         group: t.Optional[str] = None,
         source_file: t.Optional[NPath] = None,
@@ -500,6 +504,7 @@ class Report(BaseReport):
             name: The document name - can include spaces, caps, symbols, etc., e.g. "Profit & Loss 2020"
             description: A high-level description for the document, this is displayed in searches and thumbnails
             source_url: A URL pointing to the source code for the document, e.g. a GitHub repo or a Colab notebook
+            visibility: "UNLISTED" (default), "PRIVATE", or "PUBLISHED" (for studio product only, ignored on enterprise)
             tags: A list of tags (as strings) used to categorise your document
             group: Group to add the report to (Teams only)
             source_file: Path of jupyter notebook file to upload
@@ -509,17 +514,52 @@ class Report(BaseReport):
 
         display_msg("Uploading report and associated data - *please wait...*")
 
-        self._upload_report(name, description, source_url, tags, group, source_file, formatting=formatting, **kwargs)
+        self._upload_report(
+            name, description, source_url, visibility, tags, group, source_file, formatting=formatting, **kwargs
+        )
 
         if open:
             webbrowser.open_new_tab(self.web_url)
 
         display_msg(
-            text=f"Report successfully uploaded at {self.web_url}, follow the link to view and share your report",
-            md=f"Report successfully uploaded, click [here]({self.web_url}) to view and share your report",
+            text=f"Report successfully uploaded at {self.web_url}, follow the link to view and share your report.",
+            md=f"Report successfully uploaded, click [here]({self.web_url}) to view and share your report.",
         )
 
-    @capture_event("CLI Report Save")
+    def _save(
+        self,
+        path: str,
+        open: bool = False,
+        name: t.Optional[str] = None,
+        author: t.Optional[str] = None,
+        formatting: t.Optional[ReportFormatting] = None,
+    ) -> str:
+
+        if not name:
+            name = Path(path).stem[:127]
+
+        local_doc, _ = self._gen_report(embedded=True, title=name)
+        report_id = self._local_writer.write(
+            local_doc,
+            path,
+            name=name,
+            formatting=formatting,
+        )
+
+        if c.config.is_anonymous:
+            display_msg(
+                text="Report saved to ./{path}. To upload and share your report, create a free Datapane account by running `{bang}datapane signup`.",
+                path=path,
+            )
+        else:
+            display_msg(text=f"Report saved to ./{path}")
+
+        if open:
+            path_uri = f"file://{osp.realpath(osp.expanduser(path))}"
+            webbrowser.open_new_tab(path_uri)
+
+        return report_id
+
     def save(
         self,
         path: str,
@@ -535,35 +575,35 @@ class Report(BaseReport):
             open: Open in your browser after creating (default: False)
             name: Name of the document (optional: uses path if not provided)
             author: The report author / email / etc. (optional)
-            formatting: Set the basic styling for your report
+            formatting: Sets the basic report styling
         """
-        self._last_saved = path
 
-        if not name:
-            name = Path(path).stem[:127]
-
-        local_doc, _ = self._gen_report(embedded=True, title=name)
-        self._local_writer.write(
-            local_doc,
-            path,
-            name=name,
-            formatting=formatting,
-        )
+        report_id = self._save(path, open, name, author, formatting)
+        capture("CLI Report Save", report_id=report_id)
 
         # feedback form
         if random.random() < 0.05:
             display_msg(
-                text="How is your experience of Datapane? Please take two minutes to answer our anonymous product survey at https://bit.ly/3lWjRlr",
-                md="How is your experience of Datapane? Please take two minutes to answer our anonymous [product survey](https://bit.ly/3lWjRlr)",
+                text="How is your experience of Datapane? Please take two minutes to answer our anonymous product survey at {url}",
+                md="How is your experience of Datapane? Please take two minutes to answer our anonymous [product survey]({url})",
+                url="https://bit.ly/3lWjRlr",
             )
 
-        if open:
-            path_uri = f"file://{osp.realpath(osp.expanduser(path))}"
-            webbrowser.open_new_tab(path_uri)
-
     @capture_event("CLI Report Preview")
-    def preview(self) -> None:
-        self.save(self._preview_file.name, open=True)
+    def preview(
+        self,
+        open: True,
+        formatting: t.Optional[ReportFormatting] = None,
+    ) -> None:
+        """Preview the report in a new browser window.
+        Can be called multiple times, refreshing the existing created preview file.
+        Pass open=False to stop openning a new tab on subsequent previews
+
+        Args:
+            open: Open in your browser after creating (default: True)
+            formatting: Sets the basic report styling
+        """
+        self._save(self._preview_file.name, open=open, formatting=formatting)
 
     def _report_status_checks(self, processed_report_doc: etree._ElementTree, embedded: bool, check_empty: bool):
         super()._report_status_checks(processed_report_doc, embedded, check_empty)
@@ -580,10 +620,10 @@ class Report(BaseReport):
             elif c.config.is_public:
                 # only nudge public users
                 if asset_blocks < 4:
-                    url = "https://docs.datapane.com/reports/blocks/layout-pages-and-selects"
                     display_msg(
-                        text=f"Your report only contains a single element - did you know you can include additional plots, tables and text in a single report? Check out {url} for more info",
-                        md=f"Your report only contains a single element - did you know you can include additional plots, tables and text in a single report? Check out [the docs]({url}) for more info",
+                        text="Your report only contains a single element - did you know you can include additional plots, tables and text in a single report? Check out {url} for more info",
+                        md="Your report only contains a single element - did you know you can include additional plots, tables and text in a single report? Check out [the docs]({url}) for more info",
+                        url="https://docs.datapane.com/reports/blocks/layout-pages-and-selects",
                     )
 
                 # has_text: bool = processed_report_doc.xpath("boolean(/Report/Main/Page//Text)")
