@@ -51,3 +51,15 @@ def get_dp_mode() -> DPMode:
 def set_dp_mode(dp_mode: DPMode) -> None:
     global __dp_mode
     __dp_mode = dp_mode
+
+
+################################################################################
+# Built-in exceptions
+def add_help_text(x: str) -> str:
+    return f"{x}\nPlease run with `dp.enable_logging()`, restart your Jupyter kernel/Python instance, and/or visit https://www.github.com/datapane/datapane to raise issue / discuss if error repeats"
+
+
+class DPError(Exception):
+    def __str__(self):
+        # update the error message with help text
+        return add_help_text(super().__str__())
